#
#  PreferenceController.py
#  ZopeEditManager
#
#  Created by Zachery Bir on 2003-10-29.
#  Copyright (c) 2003 Zope Corporation. All rights reserved.
#

# library imports
import os, re
from objc import YES, NO
from PyObjCTools import NibClassBuilder

# import needed classes/functions from Foundation
from Foundation import *

# import Nib loading functionality from AppKit
from AppKit import *

# local product imports

# create ObjC classes as defined in Preferences.nib
NibClassBuilder.extractClasses("Preferences")
class PreferenceController(NibClassBuilder.AutoBaseClass):
    """
    The Controller object for managing various preference variables.
    """
    def prepareForEditing(self, sender):
        row = sender.clickedRow()
        _helper_apps_keys = list(self._helper_apps.keys())
        _helper_apps_keys.sort()
        type = _helper_apps_keys[row]
        
        self.type_value.setStringValue_(type)
        self.extension_value.setStringValue_(self._helper_apps[type].get(
                                                            'extension',
                                                            ''))
        self.editor_value.setStringValue_(self._helper_apps[type].get('editor',
                                                                    ''))

    def newHelper_(self, sender):
        self.type_value.setStringValue_('')
        self.extension_value.setStringValue_('')
        self.editor_value.setStringValue_('')

    def editHelper_(self, sender):
        _type_value = self.type_value.stringValue()
        _extension_value = self.extension_value.stringValue()
        _editor_value = self.editor_value.stringValue()
        if _type_value:
            self._helper_apps[_type_value] = {'extension' : _extension_value,
                                              'editor' : _editor_value}
            self.helper_apps.reloadData()
        self.newHelper_(sender)

    def removeHelper_(self, sender):
        row = self.helper_apps.selectedRow()
        _helper_apps_keys = list(self._helper_apps.keys())
        _helper_apps_keys.sort()
        type = _helper_apps_keys[row]
        del self._helper_apps[type]
        self.helper_apps.reloadData()
        self.newHelper_(sender)

    def savePreferences_(self, sender):
        self.sud.setBool_forKey_(bool(self._cleanup_files), 'cleanup_files')
        self.sud.setBool_forKey_(bool(self._confirm_on_finish),
                                 'confirm_on_finish')
        self.sud.setBool_forKey_(bool(self._use_webdav_locks), 'use_locks')
        self.sud.setBool_forKey_(bool(self._always_borrow_webdav_locks),
                                 'always_borrow_locks')
        self.sud.setFloat_forKey_(self._save_interval, 'save_interval')
        self.sud.setObject_forKey_(self._helper_apps, 'helper_apps')

        self.sud.synchronize()
        self.close()

    def setCleanupFiles_(self, sender):
        self._cleanup_files = self.cleanup_files.state()

    def setConfirmOnFinish_(self, sender):
        self._confirm_on_finish = self.confirm_on_finish.state()

    def setSaveInterval_(self, sender):
        self._save_interval = self.save_interval.floatValue()

    def setUseWebDAVLocks_(self, sender):
        self._use_webdav_locks = self.use_webdav_locks.state()
        if self._use_webdav_locks:
            self.always_borrow_webdav_locks.setEnabled_(YES)
        else:
            self.always_borrow_webdav_locks.setState_(0)
            self.always_borrow_webdav_locks.setEnabled_(NO)

    def setAlwaysBorrowWebDAVLocks_(self, sender):
        self._always_borrow_webdav_locks = self.always_borrow_webdav_locks.state()

    def windowDidLoad(self):
        self.cleanup_files.setState_(self._cleanup_files and 1 or 0)
        self.confirm_on_finish.setState_(self._confirm_on_finish and 1 or 0)
        self.save_interval.setFloatValue_(self._save_interval and
                                          self._save_interval or 0.0)
        self.use_webdav_locks.setState_(self._use_webdav_locks and 1 or 0)
        self.always_borrow_webdav_locks.setState_(
                                        self._always_borrow_webdav_locks
                                        and 1 or 0)
        self.helper_apps.setAction_('prepareForEditing')

    def numberOfRowsInTableView_(self, tableView):
        return len(self._helper_apps)

    def tableView_objectValueForTableColumn_row_(self, tableView,
                                                 aColumn, aRow):
        _helper_apps_keys = list(self._helper_apps.keys())
        _helper_apps_keys.sort()
        type = _helper_apps_keys[aRow]
        ident = aColumn.identifier()
        if ident == 'type':
            return type
        return self._helper_apps[type].get(ident, '')

    def selectionShouldChangeInTableView_(self, tableView):
        return YES

    def init(self):

        self = self.initWithWindowNibName_("Preferences")

        self.sud = NSUserDefaults.standardUserDefaults()
        self._cleanup_files = self.sud.boolForKey_('cleanup_files')
        self._confirm_on_finish = self.sud.boolForKey_('confirm_on_finish')
        self._save_interval = self.sud.floatForKey_('save_interval')
        self._use_webdav_locks = self.sud.boolForKey_('use_locks')
        self._always_borrow_webdav_locks = self.sud.boolForKey_(
                                                'always_borrow_locks')
        self._helper_apps = self.sud.dictionaryForKey_('helper_apps')

        return self

